version 4.0
"
"  ~/.vimrc  Timo Felbinger, 1999
"
"  A complete .vimrc fitting my personal preferences.  You might
"  try to use it if you do not yet have one. If you are already
"  using Vim and just want Mathematica related stuff, you should
"  look into sample.macros instead.

" customize options:
"
set secure
set noexrc
set nomodeline
set modelines=0

set number
set expandtab
set magic
set nocompatible

set ruler
set shell=bash
set autoindent
set title
set noremap
set nowrap
set sidescroll=1
set ignorecase
set hidden
set shortmess=a
set visualbell
set hlsearch

"  don't create swap files:
set uc=0

set dir=$TEMP

" ************ don't activate this: there is still some bug in vim-5.4g's implementation of statusline!
" ***** set statusline=%n\ %F%M%R%H\ %{PrintIfVarExists('b:typeoffile',b:typeoffile)}%=%B\ %l/%L,%c%V\ %p%%
" ***** set laststatus=2



"  comma: initiates special motions and non-modifying actions
"  (most of them depend on file type):

noremap ,h :noh<CR>


"  semicolon: initiates text/file-modifying operations:
"

" write buffer to disk:
"
nnoremap ;; :w<CR>

" write to and read from temporary file:
nnoremap ;w :w! ~/b<CR>
vnoremap ;w :w! ~/b<CR>
cnoremap <c-w> w! ~/b<CR>
nnoremap ;r :r ~/b<CR>

" remove trailing spaces:
"
nnoremap ;<Space> mt:.s/ *$//<CR>`t

" remove a pair of matching braces, parentheses, brackets:
"
nnoremap ;x% v%`>i l"_d? *<CR>"_x`<"_d/$\\|[^ ]<CR>:noh<CR>


" windows and buffers can be switched with the usual hjkl keys:
"
nnoremap <c-j> <c-w>j
nnoremap <c-k> <c-w>k
nnoremap <c-h> :bp<CR>
nnoremap <c-l> :bn<CR>

" allow cursor movements in command line and insert mode:
"
cnoremap <c-k> <Up>
cnoremap <c-j> <Down>
cnoremap <c-j> <Left>
cnoremap <c-l> <Right>

cnoremap <c-w> <c-right>
cnoremap <c-b> <c-left>

inoremap <c-k> <Space><Esc>xki
inoremap <c-j> <Space><Esc>xji
inoremap <c-h> <Esc>i
inoremap <c-l> <Space><Esc>xa


" useful mappings for space and backspace:
"
" in normal mode...
nnoremap <Space> i l
nnoremap <BS> hx
" ... and in visual mode:
vnoremap <Space> :s/^/ /<CR>gv
vnoremap  :s/^ //<CR>gv

" we want to keep the visual when shifting left or right:
"
vnoremap > >gv
vnoremap < <gv


" other settings for normal mode:
"

"  splitting & joining in normal mode: avoid trailing spaces,
"  do indentation correctly:
"
nnoremap 
 ixi
^x:.-1s/ *$//<CR>j^
nnoremap J :.s/ \=$/ /<CR>$J

" exchange semantics of these strange little thingies ` and '
" (semantics of ` is more useful yet the key is harder to reach!):
"
nnoremap ` '
nnoremap ' `

" insert one single character:
"
nnoremap  i r

" display buffer list and accept buffer number to switch to:
"
nnoremap \ :ls<CR>:e #

" abbreviations:
"
nnoremap { [{
nnoremap } ]}

" space in operator-pending mode moves to end of WORD (so that d<SPACE>
" will delete the WORD but, unlike dW, not the space behind it):
"
onoremap <SPACE> /\W<CR>

" leave insert/replace mode without moving the cursor left, unless
" we are beyond the end of the line:
"
inoremap <ESC> <ESC>ax<ESC>x



" allow for common typos:
"
com! W w
com! Wq wq
com! Q q
com! QQ qa!

" remove all trailing space (they can be deadly eg in printcaps on some systems):
"
com! Ceol %s/ \+$//



"  file type dependend settings:
"    - we analyze the file type when reading a buffer; this will
"      raise a flag if a type is recognized;
"    - we check for this flag when entering a buffer and activate
"      proper settings; this set the variable 'b:setting' which can
"      be displayed in the status line.

"  a function similar to VarExists(), which is suggested in the Vim docs:
"
function PrintIfVarExists( var, val )
 if exists( a:var) | return a:val | else | return '' | endif
endfunction

function ActivateSettings()
  if ! exists('b:typeoffile')
    let b:typeoffile = 'unknown'
  endif
  if b:typeoffile == 'vma'
    call VmaSettings()
  elseif b:typeoffile == 'c'
    call CSettings()
  elseif b:typeoffile == 'tex'
    call TeXSettings()
  endif
endfunction

autocmd BufEnter * call ActivateSettings()

function BufReadAny()
  let b:typeoffile = 'unknown'
  let b:settings = 'none'
endfunction

function CheckVma()
  if( match( getline(1), "^#!\\(/.\\+/\\)\\=vma version 1\\.0$" ) > -1 )
    let b:typeoffile = 'vma'
    call VmaSettings()
  else
    echo "CheckVma: Nope, this isn't a .vma file, although it has extension .vma!"
    let b:typeoffile = 'unknown'
  endif
endfunction

function CheckC()
  let b:typeoffile = 'c'
  call CSettings()
endfunction

function CheckTeX()
  let b:typeoffile = 'tex'
  call TeXSettings()
endfunction

autocmd BufReadPost * call BufReadAny()
autocmd BufReadPost *.vma call CheckVma()
autocmd BufReadPost *.{c,h,cc,hh,c?,h?} call CheckC()
autocmd BufReadPost *.tex call CheckTeX()

function CSettings()
  noremap ,] /^}<CR>
  noremap ,[ ?^\h.*{$<CR>
  "  enter more useful stuff here
endfunction

function TeXSettings()
  "  enter useful stuff here
endfunction



"
"  Mathematica related stuff:
"

function VmaSettings()

  "  find the next input/include block down:
  noremap ,/i $/^\s*i\(nclude\)\=\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>

  "  find the next input/include block up:
  noremap ,?i 0?^\s*i\(nclude\)\=\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>

  "  find the input/include block of the current cell:
  noremap ,i  $?^\s*i\(nclude\)\=\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>

  "  find previous tagged line:
  noremap ,[  0?^\s*\h\w*\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>

  "  find next tagged line:
  noremap ,]  $/^\s*\h\w*\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>

  "  evaluate current block:
  nnoremap ;e $?^\s*i\(nclude\)\=\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>:call mma_eval(line("."))<CR>

  "  evaluate visual selection:
  vnoremap ;e <ESC>:call mma_eval_range(line{"'<"),line("'>"))

  "  insert new input tag:
  noremap ;i  o<ESC>ii[0]<ESC>

  "  insert separator line:
  noremap ;s  o<ESC>68i-<ESC>

  "  delete current block:
  noremap ;d  $?^\s*\h\w*\[-\=\d\+\(\.\{3}-\=\d\+\)\=\(,\d\+\(\.\{3}\d\+\)\=\)\=]\( .*\)\=$<CR>:call mma_delete_block(line("."))<CR>

  com! Mma call LaunchMma()
  com! KillMma call kill("mma")
  com! -range In call mma_eval_range( <line1>, <line2> )
  com! Abort call mma_abort()
  com! Stopit call mma_purge_output()

  let b:settings='vma'
endfunction


function LaunchMma()
  call mma_reset()
  call fork("mma","ml2vim")
  call read_async("mma","mma")
endfunction



